@@ -172,6 +172,7 @@ class Config:
 		self._tool_stages = dict()
 		self._target_pkgs = dict()
 		self._tasks = dict()
+		self._site_archs = set()
 
 		self._bootstrap_path = os.path.join(path,
 				os.path.dirname(os.readlink(os.path.join(path, 'bootstrap.link'))))
@@ -195,6 +196,16 @@ class Config:
 		self._parse_yml(os.path.join(path, os.readlink(os.path.join(path, 'bootstrap.link'))),
 				self._root_yml)
 
+		# Collect all architectures that this build uses.
+		for tool in self._tool_pkgs.values():
+			arch = tool.architecture
+			if arch != 'noarch':
+				self._site_archs.add(arch)
+		for pkg in self._target_pkgs.values():
+			arch = pkg.architecture
+			if arch != 'noarch':
+				self._site_archs.add(arch)
+
 	def _parse_yml(self, current_path, current_yml,
 			filter_sources = None, filter_tools = None, filter_pkgs = None, filter_tasks = None):
 		if 'imports' in current_yml and isinstance(current_yml['imports'], list):
@@ -319,6 +330,10 @@ class Config:
 			return None
 		return self._site_yml['container'].get('runtime')
 
+	@property
+	def site_architectures(self):
+		return self._site_archs
+
 	@property
 	def source_root(self):
 		return os.path.join(os.getcwd(),
@@ -1311,7 +1326,14 @@ class TargetPackage(RequirementsMixin):
 					'-r', self._cfg.sysroot_dir,
 					self.name
 				], env=environ)
-				if b'state: installed' not in out.splitlines():
+				valid_state = False
+				for line in out.splitlines():
+					if not line.startswith(b'state:'):
+						continue
+					if line == b'state: installed' or line == b'state: unpacked':
+						valid_state = True
+					break
+				if not valid_state:
 					return ItemState(missing=True)
 				return ItemState()
 			except subprocess.CalledProcessError:
@@ -2185,59 +2207,73 @@ def pack_pkg(cfg, pkg, reproduce=False):
 	version = pkg.compute_version(override_rolling_id=actual_rolling_id)
 
 	if cfg.use_xbps:
+		from . import xbps_utils as _xbps_utils
 		_util.try_mkdir(cfg.xbps_repository_dir)
 
 		output = subprocess.DEVNULL
 		if verbosity:
 			output = None
 
-		environ = os.environ.copy()
-		_util.build_environ_paths(environ, 'PATH',
-				prepend=[os.path.join(_util.find_home(), 'bin')])
+		with tempfile.TemporaryDirectory() as pack_dir:
+			installtree(pkg.staging_dir, pack_dir)
 
-		args = ['xbps-create', '-A', pkg.architecture,
-			'-s', pkg.name,
-			'-n', '{}-{}'.format(pkg.name, version),
-			'-D', pkg.xbps_dependency_string()
-		]
-
-		metadata = pkg._this_yml.get('metadata', dict())
-		if 'summary' in metadata:
-			args += ['--desc', metadata['summary']]
-		if 'description' in metadata:
-			args += ['--long-desc', metadata['description']]
-		if 'spdx' in metadata:
-			args += ['--license', metadata['spdx']]
-		if 'website' in metadata:
-			args += ['--homepage', metadata['website']]
-		if 'maintainer' in metadata:
-			args += ['--maintainer', metadata['maintainer']]
-		if 'categories' in metadata:
-			args += ['--tags', ' '.join(metadata['categories'])]
-
-		args += [pkg.staging_dir]
-
-		xbps_file = '{}-{}.{}.xbps'.format(pkg.name, version, pkg.architecture)
-
-		if not reproduce:
-			_util.log_info("Running {}".format(args))
-			subprocess.call(args, env=environ, cwd=cfg.xbps_repository_dir, stdout=output)
-
-			args = ['xbps-rindex', '-fa',
-					os.path.join(cfg.xbps_repository_dir, xbps_file)
-			]
+			install_sh = _xbps_utils.compose_xbps_install(cfg, pkg)
+			if install_sh:
+				with open(os.path.join(pack_dir, 'INSTALL'), 'wt') as f:
+					f.write(install_sh)
 
+			# The directory is now prepared, call xbps-create.
 			environ = os.environ.copy()
 			_util.build_environ_paths(environ, 'PATH',
 					prepend=[os.path.join(_util.find_home(), 'bin')])
-			environ['XBPS_ARCH'] = pkg.architecture
+
+			args = ['xbps-create', '-A', pkg.architecture,
+				'-s', pkg.name,
+				'-n', '{}-{}'.format(pkg.name, version),
+				'-D', pkg.xbps_dependency_string()
+			]
+
+			metadata = pkg._this_yml.get('metadata', dict())
+			if 'summary' in metadata:
+				args += ['--desc', metadata['summary']]
+			if 'description' in metadata:
+				args += ['--long-desc', metadata['description']]
+			if 'spdx' in metadata:
+				args += ['--license', metadata['spdx']]
+			if 'website' in metadata:
+				args += ['--homepage', metadata['website']]
+			if 'maintainer' in metadata:
+				args += ['--maintainer', metadata['maintainer']]
+			if 'categories' in metadata:
+				args += ['--tags', ' '.join(metadata['categories'])]
+
+			args += [pack_dir]
+
+			xbps_file = '{}-{}.{}.xbps'.format(pkg.name, version, pkg.architecture)
 
 			_util.log_info("Running {}".format(args))
-			subprocess.call(args, env=environ, stdout=output)
+			if not reproduce:
+				subprocess.call(args, env=environ, cwd=cfg.xbps_repository_dir, stdout=output)
+			else:
+				subprocess.call(args, env=environ, cwd=cfg.package_out_dir, stdout=output)
+
+		if not reproduce:
+			rindex_archs = [pkg.architecture]
+			if pkg.architecture == 'noarch':
+				rindex_archs = cfg.site_architectures
+			for arch in rindex_archs:
+				args = ['xbps-rindex', '-fa',
+						os.path.join(cfg.xbps_repository_dir, xbps_file)
+				]
+
+				environ = os.environ.copy()
+				_util.build_environ_paths(environ, 'PATH',
+						prepend=[os.path.join(_util.find_home(), 'bin')])
+				environ['XBPS_ARCH'] = arch
+
+				_util.log_info("Running {} ({})".format(args, arch))
+				subprocess.call(args, env=environ, stdout=output)
 		else:
-			_util.log_info("Running {}".format(args))
-			subprocess.call(args, env=environ, cwd=cfg.package_out_dir, stdout=output)
-
 			if not filecmp.cmp(os.path.join(cfg.package_out_dir, xbps_file),
 					os.path.join(cfg.xbps_repository_dir, xbps_file),
 					shallow=False):
@@ -2257,12 +2293,26 @@ def install_pkg(cfg, pkg):
 		if verbosity:
 			output = None
 
+		# noarch pkgs use the first "true" architecture.
+		# TODO: this should be dependent on the sysroot that we are installing into.
+		effective_arch = pkg.architecture
+		if pkg.architecture == 'noarch':
+			effective_arch = list(cfg.site_architectures)[0]
+
 		environ = os.environ.copy()
 		_util.build_environ_paths(environ, 'PATH',
 				prepend=[os.path.join(_util.find_home(), 'bin')])
-		environ['XBPS_ARCH'] = pkg.architecture
+		environ['XBPS_ARCH'] = effective_arch
 
-		args = ['xbps-install', '-fy',
+		# Work around xbps: https://github.com/void-linux/xbps/issues/408
+		args = ['xbps-remove', '-Fy',
+			'-r', cfg.sysroot_dir,
+			pkg.name
+		]
+		_util.log_info("Running {}".format(args))
+		subprocess.call(args, env=environ, stdout=output)
+
+		args = ['xbps-install', '-fyU',
 			'-r', cfg.sysroot_dir,
 			'--repository', cfg.xbps_repository_dir,
 			pkg.name
@@ -2284,10 +2334,15 @@ def pull_pkg_pack(cfg, pkg):
 
 	_util.try_mkdir(cfg.xbps_repository_dir)
 
+	# noarch pkgs use the first "true" architecture.
+	effective_arch = pkg.architecture
+	if pkg.architecture == 'noarch':
+		effective_arch = list(cfg.site_architectures)[0]
+
 	# Download the repodata file.
-	rd_path = os.path.join(cfg.xbps_repository_dir, 'remote-{}-repodata'.format(pkg.architecture))
-	rd_url = urllib.parse.urljoin(repo_url + '/', '{}-repodata'.format(pkg.architecture))
-	_util.log_info('Downloading {}-repodata from {}'.format(repo_url, pkg.architecture))
+	rd_path = os.path.join(cfg.xbps_repository_dir, 'remote-{}-repodata'.format(effective_arch))
+	rd_url = urllib.parse.urljoin(repo_url + '/', '{}-repodata'.format(effective_arch))
+	_util.log_info('Downloading {}-repodata from {}'.format(effective_arch, repo_url))
 	_util.interactive_download(rd_url, rd_path)
 
 	# Find the package within the repodata's index file.
@@ -2307,17 +2362,21 @@ def pull_pkg_pack(cfg, pkg):
 	if verbosity:
 		output = None
 
-	args = ['xbps-rindex', '-fa',
-			os.path.join(cfg.xbps_repository_dir, xbps_file)
-	]
+	rindex_archs = [pkg.architecture]
+	if pkg.architecture == 'noarch':
+		rindex_archs = cfg.site_architectures
+	for arch in rindex_archs:
+		args = ['xbps-rindex', '-fa',
+				os.path.join(cfg.xbps_repository_dir, xbps_file)
+		]
 
-	environ = os.environ.copy()
-	_util.build_environ_paths(environ, 'PATH',
-			prepend=[os.path.join(_util.find_home(), 'bin')])
-	environ['XBPS_ARCH'] = pkg.architecture
+		environ = os.environ.copy()
+		_util.build_environ_paths(environ, 'PATH',
+				prepend=[os.path.join(_util.find_home(), 'bin')])
+		environ['XBPS_ARCH'] = arch
 
-	_util.log_info("Running {}".format(args))
-	subprocess.call(args, env=environ, stdout=output)
+		_util.log_info("Running {} ({})".format(args, arch))
+		subprocess.call(args, env=environ, stdout=output)
 
 def run_task(cfg, task):
 	tools_required = []
